﻿#----------------------------- modify $address = "192.168.32.103" to match the IP address of your K-Ducer.
#----------------------------- the K-Ducer unit should be set to MODBUS TCP protocol with static IP address (Menu>General Settings) 
#the following function initializes the connection and returns a network stream
$address = "192.168.32.103"
function Initialize {
    Param ([System.Net.Sockets.TcpClient] $tcpConnection = $NULL)
    if($NULL -ne $tcpConnection)
    {
        try { $tcpConnection.Close() }
        catch {}
        try { $tcpConnection.Dispose() }
        catch {}
    }
    $tcpConnection = New-Object System.Net.Sockets.TcpClient
    $tcpConnection.NoDelay = $TRUE
    $tcpConnection.ReceiveTimeout = 1000
    $tcpConnection.SendTimeout = 1000
    $tcpConnection.LingerState = New-Object System.Net.Sockets.LingerOption ($TRUE, 0)
    return $tcpConnection
}

#the function below sends a modbus request and receives the response
function Send-ModbusRequest {
    Param ([System.Net.Sockets.NetworkStream] $tcpStream, [array] $cmd)

    $length = [bitconverter]::GetBytes([convert]::ToInt16($cmd.length + 1))
    [array]::reverse($length)

    $data = 0x00, 0xFF, # Transaction Identifier
        0x00, 0x00 # Protocol Identifier : 0 = Modbus

    $data = $data + $length + 0x00 + $cmd

    [byte[]] $buffer = @(0) * 1024

    $tcpStream.Write($data,0,$data.length)
    $tcpStream.Flush()
    $size = $tcpStream.Read($buffer,0,$buffer.length)

    $result = @(0) * $size

    [array]::copy($buffer,$result,$size)

    #$tcpConnection.Close()

    return $result
}

#the function below requests the last screwdriving result
function Read-Screwdriver-Result {
    Param ( [System.Net.Sockets.NetworkStream] $tcpStream )

    [byte[]] $cmd = @(0x04)
    $cmd = $cmd + 0x00 +  0x00 + 0x00 + 0x7D #0x7D = 125 = 125 registers to read starting at address 0 (PrintString, per K-Ducer ModBus Map)

    $response = Send-ModbusRequest -tcpStream $tcpStream -cmd $cmd

    if($response[5] -eq 0x3){
        "Error retrieving results. ModBus exception code: " + $response[-1]
    }
    else{
        $resp = [System.Text.Encoding]::ASCII.GetString($response[9..1024])
        if($resp -eq ""){
            ;#"No result available"
        }
        return $resp
    }
}

#if you prefer a different file naming system, modify the following function
#the function below prints the string, and appends it to the specified file if it exists. if the file does not exists, it creates it.
function Show-And-Store-Screwdriving-Result
{
    Param ( [string] $screwdrivingResult )

    $screwdrivingResult #print the result to screen. comment this line if you don't want to print to screen.
    $filename = $(Get-Date -f yyyyMMdd) + ".txt"
    $path = ".\" + $filename
    if( Test-Path $path )
    {
        Add-Content -Path $path -Value $screwdrivingResult
    }
    else {
        New-Item -Path . -Name $filename -ItemType "file"
        Add-Content -Path $path -Value $screwdrivingResult
    }
    "Saved to: " + $pwd + "\" + $filename 
}

#change the following welcome message if needed
"This script will check for new screwdriving results every 300ms`nWhen a new result is found, it is printed to screen and appended to a file."

#main loop: initializes the TCP connection, and starts the sub-loop
$resp_old = ""
$resp_new = ""
$stream = $NULL
$conn = $NULL
while ($TRUE)
{
    try
    {
        $conn = Initialize $conn
        Start-Sleep 3
        $conn.Connect($address, 502)
        $stream = $conn.GetStream()
        if($NULL -eq $stream) { throw }
        "Connected to:" + $address
    }
    catch
    {
        "Failed to connect to: " + $address + "`nAttempting again in 3 seconds..."
        continue
    }
    #sub-loop: check if a new screwdriving result is available, print and save if it is
    while ($TRUE)
    {
        try
        {
            $resp_new = $(Read-Screwdriver-Result -tcpStream $stream).replace("`n","").replace("`r","")
            if( $resp_old -eq $resp_new )
            {
                Start-Sleep 0.3 #result is not new
            }
            else
            {
                $resp_old = $resp_new
                Show-And-Store-Screwdriving-Result $resp_new
                Start-Sleep 0.3
            }
        }
        catch
        {
            "Failed to retrieve the screwdriving result. Attempting to reconnect in 3 seconds..."
            Start-Sleep 3
            break
        }
    }
}
