#example of retreiving screwdriving results via MODBUS remote communication with K-Ducer
#uses python3 with pymodbustcp library from https://pypi.org/project/pyModbusTCP/
#install with pip install pyModbusTCP

from pyModbusTCP.client import ModbusClient
import time

address_offset = -1 #per modbus specifications: MODBUS addresses are indexed at 1, but in the MODBUS PDU, they are indexed at 0
#some MODBUS implementations, for example Siemens Step7 PLCs, automatically add this offset
kdu_ip_address = '192.168.32.103' #the IP address of the K-Ducer unit. Set via Menu > General Settings > Communication Protocol

def convert_kducer_printstring(output_from_function_read_input_registers_0_125):
    #input: list of integers exactly as received from K-DUCER response to read_input_registers(reg_addr=0, reg_nb=125)
    #returns: string containing screwdriving results
    #this is necessary because pyModbusTCP assumes all responses are integers, but the K-DUCER response is an ASCII coded string
    #must convert each integer to two bytes in big endian (modbus uses big endian), then decode as ASCII
    if output_from_function_read_input_registers_0_125 == None or type(output_from_function_read_input_registers_0_125) != type(['']) or len(output_from_function_read_input_registers_0_125) == 0:
        print('Input must be the list returned by the pyModbusTCP function: read_input_registers(reg_addr=0, reg_nb=125). Input was: {}'.format(output_from_function_read_input_registers_0_125))
        return ''
    return b''.join([i.to_bytes(2, byteorder='big') for i in output_from_function_read_input_registers_0_125]).decode(encoding='ascii',errors='ignore').replace('\00','').replace('\r\n','\n')

def continously_check_screwdriving_results():
    exampleMBClient = ModbusClient(host=kdu_ip_address, port=502, auto_open=True, auto_close=False, timeout=3) #instantiate a MODBUS client. auto_close must be False for REMOTE_LEVER control
    exampleMBClient.debug(False) #use True if you want the script to print all the MODBUS packets and debug informantion on the console
    
    old_response = exampleMBClient.read_input_registers(reg_addr= address_offset+1, reg_nb= 125)
    screwdriving_results_string = convert_kducer_printstring(old_response)
    print('Last result from K-Ducer with IP {}:\n{}'.format(exampleMBClient.host(),screwdriving_results_string))

    while 1:
        time.sleep(0.2) #check for a new result every 200ms. Place this loop in a thread to perform other tasks asynchronously
        new_response = exampleMBClient.read_input_registers(reg_addr= address_offset+1, reg_nb= 125)
        if new_response == old_response:
            continue #not a new result, skip rest of loop
        old_response = new_response
        screwdriving_results_string = convert_kducer_printstring(new_response)
        print('New result from K-Ducer with IP {}:\n{}'.format(exampleMBClient.host(),screwdriving_results_string))
    
if __name__ == "__main__":
    continously_check_screwdriving_results()